/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.jmx;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.DeploymentServiceMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of the JMX deployment service.
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsDeploymentServiceClient extends PetalsAbstractServiceClient {

    /**
     * Service assembly states
     */
    public final static class State {

        public final static String STARTED = "Started";

        public final static String STOPPED = "Stopped";

        public final static String SHUTDOWN = "Shutdown";

    }

    /**
     * Action : deploy
     */
    private static final String DEPLOY = "deploy";

    /**
     * Action : undeploy
     */
    private static final String UNDEPLOY = "undeploy";

    /**
     * Action : undeployAllServiceAssemblies
     */
    private static final String UNDEPLOY_ALL_SAS = "undeployAllServiceAssemblies";

    /**
     * Action : force undeploy
     */
    private static final String FORCE_UNDEPLOY = "forceUndeploy";

    /**
     * Action : start
     */
    private static final String START = "start";

    /**
     * Action : startAllServiceAsssemblies
     */
    private static final String START_ALL_SAS = "startAllServiceAssemblies";

    /**
     * Action : stop
     */
    private static final String STOP = "stop";

    /**
     * Action : stopAllServiceAsssemblies
     */
    private static final String STOP_ALL_SAS = "stopAllServiceAssemblies";

    /**
     * Action : shutdown
     */
    private static final String SHUTDOWN = "shutDown";

    /**
     * Action : shutdownAllServiceAsssemblies
     */
    private static final String SHUTDOWN_ALL_SAS = "shutdownAllServiceAsssemblies";

    /**
     * Action : getDeployedServiceAssemblies
     */
    private static final String GET_DEPLOYED_SAS = "getDeployedServiceAssemblies";

    /**
     * Action : getDeployedServiceAssembliesForComponent
     */
    private static final String GET_DEPLOYED_SAS_FOR_COMPONENT = "getDeployedServiceAssembliesForComponent";

    /**
     * Action : getServiceAssemblyDescriptor
     */
    private static final String GET_SERVICE_ASSEMBLY_DESCRIPTOR = "getServiceAssemblyDescriptor";

    /**
     * Action : getServiceUnitForServiceAssembly
     */
    private static final String GET_SERVICE_UNIT_FOR_SA = "getServiceUnitForServiceAssembly";

    /**
     * Action : getState
     */
    private static final String GET_STATE = "getState";

    private final static String DEPLOYMENT_SERVICE_JMX_NAME = "Deployment";

    /**
     * Creates a new instance of {@link PetalsDeploymentServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public PetalsDeploymentServiceClient(final String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, DEPLOYMENT_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new DeploymentServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (IOException e) {
            throw new DeploymentServiceErrorException(e);
        }
    }

    /**
     * Deploy a service assembly.
     * 
     * @see DeploymentServiceMBean#deploy(String)
     * @param packageURL
     * @throws PerformActionErrorException
     */
    public String deploy(final URL packageURL) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { packageURL.toString() };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(DEPLOY, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Undeploy a service assembly.
     * 
     * @see DeploymentServiceMBean#undeploy(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public String undeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(UNDEPLOY, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Undeploy all service assemblies.
     * 
     * @throws PerformActionErrorException
     */
    public String[] undeployAllServiceAssemblies(boolean force)
            throws DeploymentServiceErrorException {
        String[] saNames;

        Object[] objects = new Object[] { force };
        String[] strings = new String[] { boolean.class.getName() };

        try {
            saNames = (String[]) this.performAction(UNDEPLOY_ALL_SAS, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return saNames;
    }

    /**
     * Force the undeployment of the service assembly.
     * 
     * @see DeploymentServiceMBean#undeploy(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws PerformActionErrorException
     */
    public String forceUndeploy(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(FORCE_UNDEPLOY, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Start a service assembly.
     * 
     * @see DeploymentServiceMBean#start(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws PerformActionErrorException
     */
    public String start(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(START, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Start all service assemblies.
     * 
     * @throws PerformActionErrorException
     */
    public String[] startAllServiceAssemblies() throws DeploymentServiceErrorException {
        String[] saNames;

        try {
            saNames = (String[]) this.performAction(START_ALL_SAS, null, null);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return saNames;
    }

    /**
     * Stop a service assembly.
     * 
     * @see DeploymentServiceMBean#stop(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws PerformActionErrorException
     */
    public String stop(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(STOP, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Stop all service assemblies.
     * 
     * @throws PerformActionErrorException
     */
    public String[] stopAllServiceAssemblies() throws DeploymentServiceErrorException {
        String[] saNames;

        try {
            saNames = (String[]) this.performAction(STOP_ALL_SAS, null, null);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return saNames;
    }

    /**
     * Shutdown a service assembly.
     * 
     * @see DeploymentServiceMBean#shutDown(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws PerformActionErrorException
     */
    public String shutdown(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(SHUTDOWN, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Shutdown all service assemblies.
     * 
     * @throws PerformActionErrorException
     */
    public String[] shutdownAllServiceAssemblies() throws DeploymentServiceErrorException {
        String[] saNames;

        try {
            saNames = (String[]) this.performAction(SHUTDOWN_ALL_SAS, null, null);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return saNames;
    }

    /**
     * Get the current state of a JBI service assembly.
     * 
     * @param serviceAssemblyName
     *            Service assembly identifier.
     * @return the current state of the JBI service assembly.
     * @throws PerformActionErrorException
     */
    public String getState(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        String result;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (String) this.performAction(GET_STATE, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Check if a service assembly is deployed.
     * 
     * @param serviceAssembly
     *            The service assembly identifier
     * @return true if the service assembly; false if the service assembly is
     *         not deploy or if <code>serviceAssembly</code> is null.
     * @throws DeploymentServiceErrorException
     */
    public boolean isServiceAssemblyDeployed(final String serviceAssembly)
            throws DeploymentServiceErrorException {
        boolean result = false;

        String[] deployedSAs = this.getDeployedServiceAssemblies();
        for (String deployedSA : deployedSAs) {
            if (deployedSA.equals(serviceAssembly)) {
                result = true;
                break;
            }
        }

        return result;
    }

    /**
     * Get deployed service assemblies.
     * 
     * @return an array of String containing all the deployed SA.
     * @throws DeploymentServiceErrorException
     */
    public String[] getDeployedServiceAssemblies() throws DeploymentServiceErrorException {
        String[] sas;

        try {
            sas = (String[]) this.performAction(GET_DEPLOYED_SAS, null, null);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return sas;
    }

    /**
     * Get service assemblies containing service units running on a component.
     * 
     * @return an array of String containing all the deployed SA for the given
     *         component.
     * @param componentName
     * @throws PerformActionErrorException
     */
    public String[] getDeployedServiceAssembliesForComponent(final String componentName)
            throws DeploymentServiceErrorException {
        String[] sas;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            sas = (String[]) this.performAction(GET_DEPLOYED_SAS_FOR_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return sas;
    }

    /**
     * Get the service assembly descriptor.
     * 
     * @return the descriptor in String form
     * @throws DeploymentServiceErrorException
     */
    public String getServiceAssemblyDescriptor(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {
        String saDescriptor;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            saDescriptor = (String) this.performAction(GET_SERVICE_ASSEMBLY_DESCRIPTOR, objects,
                    strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return saDescriptor;
    }

    /**
     * Get Service Unit embedded in the given Service Assembly.
     * 
     * @param serviceAssemblyName
     * @return an array of SU names
     * @throws DeploymentServiceErrorException
     */
    public String[] getServiceUnitForServiceAssembly(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {
        String[] suNames;

        Object[] objects = new Object[] { serviceAssemblyName };
        String[] strings = new String[] { String.class.getName() };

        try {
            suNames = (String[]) this.performAction(GET_SERVICE_UNIT_FOR_SA, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type");
        }

        return suNames;

    }

}
