/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.jmx;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.InstallationServiceMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of the JMX installation service.
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsInstallationServiceClient extends PetalsAbstractServiceClient {

    /**
     * Action : loadNewInstaller
     */
    private static final String LOAD_NEW_INSTALLER = "loadNewInstaller";
    
    /**
     * Action : loadInstaller
     */
    private static final String LOAD_INSTALLER = "loadInstaller";
    

    /**
     * Action : loadNewInstaller
     */
    private static final String UNLOAD_INSTALLER = "unloadInstaller";

    /**
     * Action : installSharedLibrary
     */
    private static final String INSTALL_SL = "installSharedLibrary";

    /**
     * Action : uninstallSharedLibrary
     */
    private static final String UNINSTALL_SL = "uninstallSharedLibrary";

    /**
     * Action : uninstallAllSharedLibrary
     */
    private static final String UNINSTALL_ALL_SL = "uninstallAllSharedLibraries";

    /**
     * Action : getInstalledSharedLibraries
     */
    private static final String GET_INSTALLED_SLS = "getInstalledSharedLibraries";

    /**
     * Action : getInstalledComponentsForSharedLibrary
     */
    private static final String GET_COMPONENTS_FOR_SL = "getInstalledComponentsForSharedLibrary";

    /**
     * Action : startComponent
     */
    private static final String START_COMPONENT = "startComponent";

    /**
     * Action : startAllComponent
     */
    private static final String START_ALL_COMPONENTS = "startAllComponents";

    /**
     * Action : stopComponent
     */
    private static final String STOP_COMPONENT = "stopComponent";

    /**
     * Action : stopAllComponent
     */
    private static final String STOP_ALL_COMPONENTS = "stopAllComponents";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_COMPONENT = "shutdownComponent";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_ALL_COMPONENTS = "shutdownAllComponents";

    /**
     * Action : uninstallAllComponents
     */
    private static final String UNINSTALL_ALL_COMPONENTS = "uninstallAllComponent";

    /**
     * Action : unloadAllComponentInstallers.
     */
    private static final String UNLOAD_ALL_COMPONENT_INSTALLERS = "unloacAllComponentInstallers";

    /**
     * JMX name
     */
    private final static String INSTALLATION_SERVICE_JMX_NAME = "Installation";

    /**
     * Creates a new instance of {@link PetalsInstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public PetalsInstallationServiceClient(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, INSTALLATION_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new InstallationServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new InstallationServiceErrorException(e);
        } catch (IOException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Load a new installer.
     * 
     * @see InstallationServiceMBean#loadNewInstaller(String)
     * @param packageURL
     * @throws InstallationServiceErrorException
     */
    public PetalsInstallerComponentClient loadNewInstaller(final URL packageURL)
            throws InstallationServiceErrorException {
        ObjectName installerName;

        Object[] objects = new Object[] { packageURL.toString() };
        String[] strings = new String[] { String.class.getName() };

        try {
            installerName = (ObjectName) this.performAction(LOAD_NEW_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return new PetalsInstallerComponentClient(this.petalsDomain, installerName,
                this.mBeanServerConnection);
    }
    
    /**
     * Load an installer.
     * 
     * @see InstallationServiceMBean#loadNewInstaller(String)
     * @param packageURL
     * @throws InstallationServiceErrorException
     */
    public PetalsInstallerComponentClient loadInstaller(final String componentName)
            throws InstallationServiceErrorException {
        ObjectName installerName;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            installerName = (ObjectName) this.performAction(LOAD_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return new PetalsInstallerComponentClient(this.petalsDomain, installerName,
                this.mBeanServerConnection);
    }

    /**
     * 
     * Unload a component installer. The component is deleted as well.
     * 
     * @see InstallationServiceMBean#unloadInstaller(String, boolean)
     * @param componentName
     * @throws InstallationServiceErrorException
     */
    public boolean unloadInstaller(final String componentName)
            throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName, true };
        String[] strings = new String[] { String.class.getName(), boolean.class.getName() };

        try {
            result = (Boolean) this.performAction(UNLOAD_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Install a shared-library.
     * 
     * @see InstallationServiceMBean#installSharedLibrary(String)
     * @return The unique name of the shared library installed.
     * @throws InstallationServiceErrorException
     */
    public String installSharedLibrary(final URL sharedLibraryURL)
            throws InstallationServiceErrorException {
        String slName;

        Object[] objects = new Object[] { sharedLibraryURL.toString() };
        String[] strings = new String[] { String.class.getName() };

        try {
            slName = (String) this.performAction(INSTALL_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slName;
    }

    /**
     * Uninstall a Shared Library.
     * 
     * @see InstallationServiceMBean#uninstallSharedLibrary(String)
     * @return true if the uninstall was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean uninstallSharedLibrary(final String sharedLibraryName)
            throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { sharedLibraryName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(UNINSTALL_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Uninstall all shared-library.
     * 
     * @return the list of the Shared Libraries names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException {
        String[] slNames;

        try {
            slNames = (String[]) this.performAction(UNINSTALL_ALL_SL, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Get all installed shared libraries. This API is not defined by JBI.
     * 
     * @return All installed shared libraries.
     * @throws InstallationServiceErrorException
     */
    public String[] getInstalledSharedLibraries() throws InstallationServiceErrorException {
        String[] slNames;

        try {
            slNames = (String[]) this.performAction(GET_INSTALLED_SLS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Get the components that use a shared library. This API is not defined by
     * JBI.
     * 
     * @return All component identifiers using the provided shared-library.
     * @throws PerformActionErrorException
     */
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName)
            throws InstallationServiceErrorException {
        String[] slNames;

        Object[] objects = new Object[] { sharedLibraryName };
        String[] strings = new String[] { String.class.getName() };

        try {
            slNames = (String[]) this.performAction(GET_COMPONENTS_FOR_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Start the component.
     * 
     * @param componentName
     * @return {@code true} if the start was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean startComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(START_COMPONENT, objects, strings);            
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Start all the component.
     * 
     * @return the list of component names that has been started
     * @throws InstallationServiceErrorException
     */
    public String[] startAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(START_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Stop the component.
     * 
     * @param componentName
     * @return {@code true} if the stop was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean stopComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(STOP_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Stop all the component.
     * 
     * @return the list of component names that has been stopped
     * @throws InstallationServiceErrorException
     */
    public String[] stopAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(STOP_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Shutdown the component.
     * 
     * @param componentName
     * @return {@code true} if the shut down was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean shutdownComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(SHUTDOWN_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Shutdown all the component.
     * 
     * @return the list of component names that has been shut down
     * @throws InstallationServiceErrorException
     */
    public String[] shutdownAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(SHUTDOWN_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Uninstall all the component.
     * 
     * @return the list of component names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(UNINSTALL_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Unload all the component installers.
     * 
     * @return the list of component names whose installers has been unloaded.
     * @throws InstallationServiceErrorException
     */
    public String[] unloadAllComponentInstallers(boolean force)
            throws InstallationServiceErrorException {
        String[] componentNames;

        Object[] objects = new Object[] { force };
        String[] strings = new String[] { boolean.class.getName() };

        try {
            componentNames = (String[]) this.performAction(UNLOAD_ALL_COMPONENT_INSTALLERS,
                    objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }
}
